# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc

from . import remote_layer_pb2 as remote__layer_dot_remote__layer__pb2


class RemoteLayerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Init = channel.unary_unary(
            "/livesync.RemoteLayer/Init",
            request_serializer=remote__layer_dot_remote__layer__pb2.InitRequest.SerializeToString,
            response_deserializer=remote__layer_dot_remote__layer__pb2.InitResponse.FromString,
        )
        self.Call = channel.unary_unary(
            "/livesync.RemoteLayer/Call",
            request_serializer=remote__layer_dot_remote__layer__pb2.CallRequest.SerializeToString,
            response_deserializer=remote__layer_dot_remote__layer__pb2.CallResponse.FromString,
        )


class RemoteLayerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Init(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Call(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_RemoteLayerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Init": grpc.unary_unary_rpc_method_handler(
            servicer.Init,
            request_deserializer=remote__layer_dot_remote__layer__pb2.InitRequest.FromString,
            response_serializer=remote__layer_dot_remote__layer__pb2.InitResponse.SerializeToString,
        ),
        "Call": grpc.unary_unary_rpc_method_handler(
            servicer.Call,
            request_deserializer=remote__layer_dot_remote__layer__pb2.CallRequest.FromString,
            response_serializer=remote__layer_dot_remote__layer__pb2.CallResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler("livesync.RemoteLayer", rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class RemoteLayer(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Init(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/livesync.RemoteLayer/Init",
            remote__layer_dot_remote__layer__pb2.InitRequest.SerializeToString,
            remote__layer_dot_remote__layer__pb2.InitResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Call(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/livesync.RemoteLayer/Call",
            remote__layer_dot_remote__layer__pb2.CallRequest.SerializeToString,
            remote__layer_dot_remote__layer__pb2.CallResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
